import argparse
import sys

import requests


from cool_project.cervices.data_output import (
    console_output_feed, console_json_output
)
from cool_project.cervices.decorators import (
    check_limit_type_value,
    intercept_errors,
    verbose_information_about_start_scrapping,
    decorator_delimiter,
)
from cool_project.cervices.print_functions import info_print, error_print
from cool_project.serializers.serializers import serialization_data
from cool_project.data_storage.working_with_storage import storage_control
from project_settings import PROGRAM_VERSION, HEADERS


class RSSParser:
    """
    Class that regulates the parsing relationship between
    the user and the site that program are trying to parse.
    """

    def __init__(self, args_for_argparse):
        """
        Init class and init argparse
        """
        argparse_params = self._init_argparse(args_for_argparse)
        self.source = argparse_params.source
        self.limit = argparse_params.limit
        self.json = argparse_params.json
        self.verbose = argparse_params.verbose
        self.date = argparse_params.date
        self.to_html = argparse_params.to_html
        self.to_pdf = argparse_params.to_pdf
        self.colorize = argparse_params.colorize
        self.serializable_data = None

    @staticmethod
    @check_limit_type_value
    def _init_argparse(args_for_argparse):
        """
        Initialization argparse and check if --version option
        is specified app should just print its version and stop.

        :return: args of argparse
        """
        parser = argparse.ArgumentParser(
            description="Pure Python command-line RSS reader."
        )
        parser.add_argument(
            "source", nargs="?", default=None,
            type=str, help="Print version info"
        )
        parser.add_argument(
            "--version", action="version",
            version=f"Version {PROGRAM_VERSION}", help="RSS URL"
        )
        parser.add_argument(
            "--json", action="store_true",
            help="Print result as JSON in stdout"
        )
        parser.add_argument(
            "--verbose", action="store_true",
            help="Outputs verbose status messages"
        )
        parser.add_argument(
            "--limit", help="Limit news topics if this parameter provided"
        )
        parser.add_argument(
            "--date", help="Take a date in %Y%m%d format. Example: 20191206"
        )
        parser.add_argument(
            "--to-html",
            help="This argument receives the path "
                 "where new HTML file will be saved."
        )
        parser.add_argument(
            "--to-pdf",
            help="This argument receives the path "
                 "where new PDF file will be saved."
        )
        parser.add_argument(
            "--colorize", action="store_true",
            help="That will print the result of the utility in colorized mode."
        )

        args, unknown = parser.parse_known_args(args=args_for_argparse)

        # If --version option is specified app should
        # just print its version and stop.
        if "—version" in unknown:
            parser.parse_args(["--version"])
        else:
            return args

    @verbose_information_about_start_scrapping
    def parsing(self):
        """
        Gets the response object and checks that it isvalid
        and call serialization_data to get serializable_data
        """
        response = self._get_html()

        if self._isvalid(response):
            serializable_data = serialization_data(
                response.text, self.limit, self.verbose, self.source
            )

            if serializable_data is None:
                error_print("Data wasn't received")
                return False

            info_print("Receiving the news was successful")
            self.serializable_data = serializable_data

    def _isvalid(self, response):
        """Check if response is valid"""
        if response is None:
            if self.verbose:
                info_print(
                    f"The program stop running with error, when it "
                    f"try to get information from {self.source!r}"
                )
            return False

        elif response.status_code == 200:
            return True
        else:
            self._check_error_status_code(response.status_code)

    def check_date_and_source(self):
        """
        Check date and source value and allowed standard start if
        source was enter and date is None, if source is None
        print error, if date was enter call storage_control.

        :return: True if source is enter and date is None
        """
        if self.date is None and self.source is not None:
            return True
        elif self.date is not None:
            storage_control(
                date=self.date, source=self.source, verbose=self.verbose,
                json=self.json, limit=self.limit, to_html=self.to_html,
                to_pdf=self.to_pdf, colorize=self.colorize
            )
        elif self.source is None:
            if self.verbose:
                info_print(f"Source is {self.source}")
            error_print("A source wasn't enter")

    def print_data_in_console(self):
        """
        Check the self.json value and
        if self.json is True - outputs json to the console
        if self.json is False - outputs news
        in a standard format to the console
        """
        if self.serializable_data is None:
            return False

        if self.json:
            if self.verbose:
                info_print("Output news in JSON format")
            console_json_output(self.serializable_data)
        else:
            if self.verbose:
                info_print("Output news in standard format")
            console_output_feed(self.serializable_data, self.colorize)

        storage_control(
            data=self.serializable_data, source=self.source,
            verbose=self.verbose, to_html=self.to_html,
            to_pdf=self.to_pdf, colorize=self.colorize
        )

    @intercept_errors
    def _get_html(self):
        """
        Executes a get request at the url specified by the user
        and check encoding of response data.

        :return response obj
        """
        response = requests.get(self.source, headers=HEADERS)

        # if site gives invalid encoding this line trying to correct it
        response.encoding = response.apparent_encoding
        return response

    def _check_error_status_code(self, status_code):
        """
        Check status code and print error message.

        :param status_code: http status code
        """
        if 400 <= status_code <= 499:
            if status_code == 404:
                error_print(f"{self.source!r}: 404 Page Not Found")
            else:
                error_print(
                    "Error seems to have been caused "
                    "by the client. Check url which you give."
                )
        elif 500 <= status_code <= 599:
            error_print("The server failed to execute a request")
        else:
            error_print(
                "Error which can't be processed because "
                "status code don't defined"
            )


@decorator_delimiter("Start Program", "Stop Program")
@intercept_errors
def start_parsing(reader):
    """Load parsing and print data"""
    if reader.check_date_and_source():
        reader.parsing()
        reader.print_data_in_console()


def main():
    """Init reader"""
    reader = RSSParser(sys.argv[1:])
    start_parsing(reader)


if __name__ == '__main__':
    main()



from pathlib import Path

from bs4 import BeautifulSoup
import pytest

from cool_project.serializers.serializers import (
    serialization_data,
    serialization_item,
    checking_the_source_is_the_rss,
    get_channel_data,
    percent_generator,
    check_limit,
)


@pytest.fixture()
def news_data():
    """fixture which read data.xml file"""
    path = Path(Path(__file__).parent, "data.xml")
    with open(path, encoding="utf-8") as file:
        data = file.read()
    return data


@pytest.fixture()
def news_data_full():
    """fixture which read full_data.xml file"""
    path = Path(Path(__file__).parent, "full_data.xml")
    with open(path, encoding="utf-8") as file:
        data = file.read()
    return data


def test_get_channel_data(soup_fix):
    """test for get_channel_data"""
    result_data = get_channel_data(soup_fix, "source")
    correct_result = {
        "channel_title": "Люди Onlíner",
        "source": "source"
    }
    assert result_data == correct_result

    result_data = get_channel_data(None, "source")
    correct_result = {
        "channel_title": None,
        "source": "source"
    }
    assert result_data == correct_result


def test_percent_generator(capsys):
    """test for percent_generator"""
    list_items = [1]
    gen = percent_generator(list_items, 5)
    list_items.append(2)
    next(gen)
    captured = capsys.readouterr()
    assert captured.out == f"[INFO] News received [1/5], " \
                           f"percent of execution program=20%\n\n"
    list_items.append(2)
    next(gen)
    captured = capsys.readouterr()
    assert captured.out == f"[INFO] News received [2/5], " \
                           f"percent of execution program=40%\n\n"
    list_items.append(2)
    next(gen)
    capsys.readouterr()
    list_items.append(2)
    next(gen)
    capsys.readouterr()
    list_items.append(2)
    next(gen)
    captured = capsys.readouterr()
    assert captured.out == f"[INFO] News received [5/5], " \
                           f"percent of execution program=100%\n\n"


@pytest.mark.parametrize("soup, verbose, source, correct_res",
                         [("soup_fix", True, "source", True),
                          ("soup_fix", False, "source", True),
                          ("", True, "source", False),
                          ("", False, "source", False)]
                         )
def test_checking_the_source_is_the_rss(
        soup, verbose, source, correct_res, request, capsys
):
    """test for checking_the_source_is_the_rss"""
    if soup:
        # This function dynamically runs a named fixture function.
        soup = request.getfixturevalue(soup)
    result = checking_the_source_is_the_rss(soup, verbose, "source")
    capsys.readouterr()
    assert result == correct_res


@pytest.mark.parametrize("limit, count_news, correct_res",
                         [(None, 10, 10),
                          (15, 10, 10),
                          (-1, 10, False),
                          (5, 10, 5)]
                         )
def test_check_limit(limit, count_news, correct_res, capsys):
    """test for check_limit"""
    result = check_limit(limit, count_news)
    capsys.readouterr()
    assert result == correct_res


def test_serialization_item(soup_fix, news_data_full):
    """test for serialization_item"""
    item = soup_fix.find("item")
    data = serialization_item(item)
    correct_res = {
        "title": "Пиневич: 40% коечного фонда "
                 "перепрофилировано под ковидных пациентов",
        "date": "Sat, 23 Oct 2021 17:51:11 +0300",
        "link": "https://people.onliner.by/2021/10/23/"
                "pereprofilirovano-pod-kovidnyx-pacientov",
        "author": None,
        "category": ["Здоровье"],
        "description": 'По всей стране в медицинских учреждениях '
                       'перепрофилировано чуть более 40% коечного фонда. '
                       'Такую цифру привел глава Министерства '
                       'здравоохранения Дмитрий Пиневич, '
                       'отметив, что это позволяет и '
                       'дальше оказывать плановую помощь '
                       'жителям Беларуси.Читать далее…',
        "more_description": None,
        "comments": None,
        "media_object": ["https://content.onliner.by/news/thumbnail/"
                         "5a7aa9c81d307b0ddc03a0f10746bffe.jpeg"],
        "extra_links": "https://people.onliner.by/2021/10/23/"
                       "pereprofilirovano-pod-kovidnyx-pacientov",
        "source_feed": None,
    }
    assert data == correct_res

    data = serialization_item(None)
    correct_res = {
        "title": None,
        "date": None,
        "link": None,
        "author": None,
        "category": None,
        "description": None,
        "more_description": None,
        "comments": None,
        "media_object": None,
        "extra_links": None,
        "source_feed": None,
    }
    assert data == correct_res

    soup = BeautifulSoup(news_data_full, "xml")
    item_full = soup.find("item")
    data_full = serialization_item(item_full)
    correct_res_full = {
        "title": "Пиневич: 40% коечного фонда перепрофилировано "
                 "под ковидных пациентов",
        "date": "Sat, 23 Oct 2021 17:51:11 +0300",
        "link": "https://people.onliner.by/2021/10/23/"
                "pereprofilirovano-pod-kovidnyx-pacientov",
        "author": "Onliner",
        "category": ["Здоровье"],
        "description": 'passs',
        "more_description": "Marissa talks to long.",
        "comments": "passs",
        "media_object": "passs",
        "extra_links": "https://people.onliner.by/2021/10/23/"
                       "pereprofilirovano-pod-kovidnyx-pacientov",
        "source_feed": ['passs', 'passs'],
    }
    assert data_full == correct_res_full


def test_serialization_data(news_data, capsys):
    """test for serialization_data"""
    data = serialization_data(news_data, 3, True, "source")
    capsys.readouterr()
    correct_res = [
        {
            "channel_title": "Люди Onlíner",
            "source": "source"
        },
        {
            "title": "Пиневич: 40% коечного фонда "
                     "перепрофилировано под ковидных пациентов",
            "date": "Sat, 23 Oct 2021 17:51:11 +0300",
            "link": "https://people.onliner.by/2021/10/23/"
                    "pereprofilirovano-pod-kovidnyx-pacientov",
            "author": None,
            "category": ["Здоровье"],
            "description": 'По всей стране в медицинских учреждениях '
                           'перепрофилировано чуть более 40% коечного фонда. '
                           'Такую цифру привел глава Министерства '
                           'здравоохранения Дмитрий Пиневич, '
                           'отметив, что это позволяет и '
                           'дальше оказывать плановую помощь '
                           'жителям Беларуси.Читать далее…',
            "more_description": None,
            "comments": None,
            "media_object": ["https://content.onliner.by/news/thumbnail/"
                             "5a7aa9c81d307b0ddc03a0f10746bffe.jpeg"],
            "extra_links": "https://people.onliner.by/2021/10/23/"
                           "pereprofilirovano-pod-kovidnyx-pacientov",
            "source_feed": None,
        }
    ]
    assert data == correct_res

    incorrect_data = f"<item></item>"
    data = serialization_data(incorrect_data, 3, True, "source")
    captured = capsys.readouterr()
    assert captured.out == "[WARNING] 'source' isn't a RSS. Please try " \
                           "to enter a correct URL\n\n[INFO] If your " \
                           "sure that this URL is correct, please check" \
                           " your URL, maybe it use old rss version and" \
                           " parser don't understand it.\n\n"



def info_print(message):
    """
    Function which print information message

    :param message: message
    """
    print(f"[INFO] {message}", end="\n\n")


def warning_print(message):
    """
    Function which print warning message

    :param message: message
    """
    print(f"[WARNING] {message}", end="\n\n")


def error_print(message):
    """
    Function which print error message

    :param message: message
    """
    print(f"[ERROR] {message}", end="\n\n")



import json

from colorama import init, Fore, Style, deinit

from cool_project.cervices.decorators import decorator_delimiter


def console_output_feed(news, colorize):
    """
    Function which print news in console in standard format.

    :param colorize: the flag which shows that need paint output
    :param news: list of dicts which contains news information
    """
    print('\n')  # line break for correct output

    channel_data = news[0]
    if colorize:
        init(autoreset=True, strip=False)
        print(
            Style.BRIGHT + Fore.GREEN + f"Feed source:",
            f"{channel_data['source']}"
        )
        print(
            Style.BRIGHT + Fore.GREEN + f"Feed:",
            Fore.MAGENTA + f"{channel_data['channel_title']}" + Style.DIM,
            end="\n\n"
        )
    else:
        print(f"Feed source: {channel_data['source']}")
        print(f"Feed: {channel_data['channel_title']}", end="\n\n")

    for item in news[1:]:
        output_feed(item, colorize)
        print()  # line break for correct output

    if colorize:
        deinit()


@decorator_delimiter("News", calls_stat=True)
def output_feed(news, colorize):
    """
    Function which processing dictionary and if value is None
    don't print pair - key, value in console. If value contain text
    print pair - key, value which contain information about news.

    :param colorize: the flag which shows that need paint output
    :param news: dict which contain information about news
    """
    for key, value in news.items():
        if value is None or value == []:
            line_break(key)
        else:
            if isinstance(value, list):
                value = rebuild_value(value)

            line_break(key)

            if colorize:
                if key == "description" or key == "more_description":
                    print(
                        Style.BRIGHT + Fore.GREEN +
                        f"{key.title().replace('_', ' ')}:",
                        f"{value.replace('&nbsp', ' ')}"
                    )
                elif key == "title":
                    print(
                        Style.BRIGHT + Fore.RED +
                        f"{key.title().replace('_', ' ')}:",
                        Style.BRIGHT + Fore.YELLOW +
                        f"{value.replace('&nbsp', ' ')}"
                    )
                else:
                    print(
                        Style.BRIGHT + Fore.CYAN +
                        f"{key.title().replace('_', ' ')}:",
                        f"{value.replace('&nbsp', ' ')}"
                    )
            else:
                print(
                    f"{key.title().replace('_', ' ')}: "
                    f"{value.replace('&nbsp', ' ')}"
                )


def line_break(key):
    """
    Print line breaks when key in dict equals description or comments

    :param key: key of dictionary
    """
    if key == "description" or key == "comments":
        print()  # line break for correct output


def rebuild_value(value):
    """
    Function rebuild value to string

    :param value: value which is a list
    :return: string
    """
    value = ", ".join(value)
    return value


def console_json_output(data):
    """
    function which processing python data to json format
    and print it to the console.

    :param data: list of dicts which contains news information
    :return: list of dicts which contains news information
    """
    json_formatted_text = json.dumps(data, indent=4, ensure_ascii=False)
    print(json_formatted_text)



from bs4 import BeautifulSoup

from cool_project.cervices.print_functions import (
    info_print, warning_print, error_print
)


def serialization_data(data, limit, verbose, source):
    """
    The function that processes the data with the soup class,
    gets the channel name and dictionaries with the news content
    and forms a list of it all.

    :param data: response text
    :param limit: limit of the news
    :param verbose: flag which means that program must print log information
    :param source: link that is used in parse print
    :return: list of the dicts which contains information about news
    """
    list_items = list()

    soup = BeautifulSoup(data, "xml")

    # finish work if get False
    if not checking_the_source_is_the_rss(soup, verbose, source):
        return None

    channel = get_channel_data(soup, source)
    list_items.append(channel)

    items = soup.find_all("item")
    count_news = len(items)
    limit = check_limit(limit, count_news)

    if verbose:
        info_print(f"Count of news: {limit}")
    gen = percent_generator(list_items, limit)

    for item in items[:limit]:
        list_items.append(serialization_item(item))

        if verbose:
            next(gen)

    return list_items


def get_channel_data(soup, source):
    """
    Find channel title and return it or None.
    Write link to source in dict.

    :param soup: BeautifulSoup object
    :param source: source or the feed
    :return: dict which contain channel_data
    """
    try:
        channel_title = soup.find("title").get_text(strip=True)
    except AttributeError:
        channel_title = None

    channel = {
        "channel_title": channel_title,
        "source": source
    }
    return channel


def percent_generator(list_items, limit):
    """
    Generator that prints information about the percent of program execution.

    :param list_items: list of dictionaries with received news
    :param limit: limit of the news
    """
    percent_of_one_items = 100 / limit
    percent_of_complete_program = percent_of_one_items
    while True:
        info_print(
            f"News received [{len(list_items) - 1}/{limit}], "
            f"percent of execution program={int(percent_of_complete_program)}%"
        )
        yield
        percent_of_complete_program += percent_of_one_items


def checking_the_source_is_the_rss(soup, verbose, source):
    """
    Find element version in xml tree and print warning if source isn't RSS.

    :param soup: BeautifulSoup object
    :param verbose: flag which means that program must print log information
    :param source: link that is used in parse print
    :return: True if source is RSS or False if source isn't RSS
    """
    try:
        soup.find("rss").get("version")
        return True
    except AttributeError:
        warning_print(
            f"{source!r} isn't a RSS. Please try to enter a correct URL"
        )
        if verbose:
            info_print(
                "If your sure that this URL is correct, please check your URL,"
                " maybe it use old rss version and parser don't understand it."
            )
        return False


def check_limit(limit, count_news):
    """
    Checked limit and if limit is None or limit > count_news,
    and check if limit <= 0,
    redefined limit to count_news.

    :param limit: limit of the news
    :param count_news: the number of news on the site
    :return: updated limit
    """
    if limit is None or limit > count_news:
        limit = count_news
    elif limit <= 0:
        error_print(
            "The limit is less than or equal to 0, news cannot be printed."
        )
        return False

    return limit


def serialization_item(item):
    """
    Function that processes the news and gets the values of an element
    to build a dictionary that will contain information about the news.

    :param item: one news
    :return: dict which contain information about news
    """
    try:
        title = item.find("title").get_text(strip=True)
        title = BeautifulSoup(title, 'html.parser')
        title = str(title)
    except AttributeError:
        title = None

    try:
        link = item.find("link").get_text(strip=True)
    except AttributeError:
        link = None

    try:
        author = item.find("author").get_text(strip=True)
    except AttributeError:
        author = None

    try:
        description_data = item.find("description").get_text(strip=True)
        soup = BeautifulSoup(description_data, 'html.parser')
        description = soup.get_text()
    except AttributeError:
        description = None

    try:
        content_encoded = item.find("content:encoded").get_text(strip=True)
        soup = BeautifulSoup(content_encoded, 'html.parser')
        content_encoded = soup.get_text()
    except AttributeError:
        content_encoded = None

    try:
        list_categories = []
        categories = item.find_all("category")

        for category in categories:
            category_content = category.get_text(strip=True)
            list_categories.append(category_content)

    except AttributeError:
        list_categories = None

    try:
        comments = item.find("comments").get_text(strip=True)
    except AttributeError:
        comments = None

    try:
        enclosure = item.find("enclosure").get("url")
    except AttributeError:
        enclosure = None

    try:
        images_list = []

        description_data = item.find("description").get_text(strip=True)
        soup = BeautifulSoup(description_data, 'html.parser')
        images = soup.find_all('img')

        for img in images:
            images_list.append(img.get("src"))

    except AttributeError:
        images_list = None

    try:
        guid = item.find("guid").get_text(strip=True)
    except AttributeError:
        guid = None

    try:
        pub_date = item.find("pubDate").get_text(strip=True)
    except AttributeError:
        pub_date = None

    try:
        source_content = item.find("source").get_text(strip=True)
    except AttributeError:
        source_content = None

    try:
        source_url = item.find("source").get("url")
    except AttributeError:
        source_url = ""

    # if description text in rss equals content_encoded
    # text - getting rid of duplicates
    if description == content_encoded:
        content_encoded = None

    # check media objects
    if images_list and enclosure:
        enclosure = images_list.append(enclosure)
    elif images_list:
        enclosure = images_list

    # check source
    if source_url and source_content:
        list_source = [source_content, source_url]
    elif source_url:
        list_source = source_url
    else:
        list_source = source_content

    item_dict = {
        "title": title,
        "date": pub_date,
        "link": link,
        "author": author,
        "category": list_categories,
        "description": description,
        "more_description": content_encoded,
        "comments": comments,
        "media_object": enclosure,
        "extra_links": guid,
        "source_feed": list_source,
    }

    return item_dict



import shutil
from math import ceil

from requests import exceptions
import pytest

from cool_project.cervices.decorators import (
    check_limit_type_value,
    intercept_errors,
    verbose_information_about_start_scrapping,
    get_data_for_print_delimiter,
    decorator_delimiter,
)


class Limit:
    """simple class for test decorator"""

    def __init__(self, limit):
        self.limit = limit


def function_for_test(value):
    """simple function for test decorator"""
    limit = Limit(value)
    return limit


def test_check_limit_type_value(capsys):
    """test for error_print"""
    dec_func = check_limit_type_value(function_for_test)
    assert dec_func(5).limit == 5

    dec_func = check_limit_type_value(function_for_test)
    assert dec_func("5").limit == 5

    dec_func = check_limit_type_value(function_for_test)
    dec_func("five")
    captured = capsys.readouterr()
    assert captured.out == "[WARNING] You must enter the number " \
                           "in --limit params.\n\n"

    dec_func = check_limit_type_value(function_for_test)
    assert dec_func(None).limit is None


def raise_exc(exc):
    """simple function for test decorator"""
    if exc is None:
        return True
    raise exc


tasks_to_try = (
    (exceptions.ConnectionError,
     "[ERROR] Connection error. Please check your URL\n\n"),
    (exceptions.MissingSchema(
        "Invalid URL 'httpslenta.r/rs': No schema supplied."
    ),
     "[ERROR] Invalid URL 'httpslenta.r/rs': No schema supplied.\n\n"),
    (Exception, "[ERROR] Unknown error\n\n")
)


@pytest.mark.parametrize("exc, value", tasks_to_try)
def test_intercept_errors(capsys, exc, value):
    """test for error_print"""
    dec_func = intercept_errors(raise_exc)
    assert dec_func(None)

    dec_func = intercept_errors(raise_exc)
    dec_func(exc)
    captured = capsys.readouterr()
    assert captured.out == value


class Verbose:
    """simple class for test decorator"""

    def __init__(self, verbose):
        self.verbose = verbose


def function_for_test_2(value):
    """simple function for test decorator"""
    return value


def test_verbose_information_about_start_scrapping(capsys):
    """test for verbose_information_about_start_scrapping"""
    dec_func = verbose_information_about_start_scrapping(function_for_test_2)
    res = dec_func(Verbose(True))
    captured = capsys.readouterr()
    assert captured.out == "[INFO] Start Scrapping\n\n" \
                           "[INFO] Stop Scrapping\n\n"
    assert res.verbose and isinstance(res.verbose, bool)

    dec_func = verbose_information_about_start_scrapping(function_for_test_2)
    res = dec_func(Verbose(False))
    assert not res.verbose and isinstance(res.verbose, bool)


def calculate_terminal_size(word):
    """
    calculated terminal size
    :param word: a word which will be printed in center of a separator
    :return: left_columns_count, right_columns_count, word
    """
    columns = shutil.get_terminal_size().columns

    if word is None:
        word = ""
    else:
        word = f" {word} "

    columns_count = columns - len(word)
    left_columns_count = ceil(columns_count / 2)
    right_columns_count = columns_count - left_columns_count
    return left_columns_count, right_columns_count, word


def test_get_data_for_print_delimiter():
    """test for get_data_for_print_delimiter"""
    word = "FINISH HIM!"
    res = get_data_for_print_delimiter(word)
    assert res == calculate_terminal_size(word)

    word = None
    res = get_data_for_print_delimiter(word)
    assert res == calculate_terminal_size(word)


def test_decorator_delimiter(capsys):
    """test for decorator_delimiter"""
    dec_func = decorator_delimiter("Fight")
    dec_func = dec_func(function_for_test_2)
    dec_func("POP")
    captured = capsys.readouterr()

    first_line = calculate_terminal_size("Fight")
    last_line = calculate_terminal_size(None)
    assert captured.out == f"{'-' * first_line[0]}{first_line[2]}" \
                           f"{'-' * first_line[1]}\n" \
                           f"{'-' * last_line[0]}{last_line[2]}" \
                           f"{'-' * last_line[1]}\n"

    dec_func = decorator_delimiter("Fight", "END", True)
    dec_func = dec_func(function_for_test_2)
    dec_func("POP")
    captured = capsys.readouterr()

    first_line = calculate_terminal_size("Fight 1")
    last_line = calculate_terminal_size("END")
    assert captured.out == f"{'-' * first_line[0]}{first_line[2]}" \
                           f"{'-' * first_line[1]}\n" \
                           f"{'-' * last_line[0]}{last_line[2]}" \
                           f"{'-' * last_line[1]}\n"



from cool_project.cervices.print_functions import info_print
from cool_project.data_storage.storage_managers import (
    FindManagerWhenEnterDate,
    FindManagerWhenEnterDateAndSource,
    DataManagerInStorageAfterParsing
)
from cool_project.conversion_to_format.conversion_to_html import convert_to_html
from cool_project.conversion_to_format.conversion_to_pdf import convertor_to_pdf


def interface_to_convert(data, to_html, to_pdf, verbose):
    """
    Interface to convert data to some format.

    :param data: a data to convert
    :param to_html: a flag
    :param to_pdf: a flag
    :param verbose: a flag
    """
    if to_html is not None:
        convert_to_html(data, to_html, verbose)

    if to_pdf is not None:
        convertor_to_pdf(data, to_pdf, verbose)


def storage_control(*, date=None, source=None, data=None, verbose=None, **kwargs):
    """
    The interface between rss_reader and data storage.
    Depending on the received data,
    call the following managers to work with the storage:

    DataManagerInStorageAfterParsing - if the data and the source
    are received, the case when data save in the storage after parsing;

    FindManagerWhenEnterDate - if date is received and source is None,
    case when a date-only search in the storage;

    FindManagerWhenEnterDateAndSource - if date and source
    are received, case when a search in the storage by date and source;

    Accepts only keyword arguments.

    :param date: the date on which you need to receive the news
    :param source: news source
    :param data: data that need to write to the storage
    :param verbose: verbose mode
    :param kwargs: --json value, --limit value or other
    :return:
    """
    # after parsing, writing a data to the storage
    if data is not None and source is not None:
        st_manager = DataManagerInStorageAfterParsing(
            source, data=data, verbose=verbose, colorize=kwargs["colorize"]
        )
        response_from_split_data_by_news = st_manager.split_data_by_news()

        if response_from_split_data_by_news is None:
            return False

        channel_data, dict_for_data_saving = response_from_split_data_by_news
        st_manager.make_dir_by_key(dict_for_data_saving)
        st_manager.control_of_exist(dict_for_data_saving, channel_data)

        interface_to_convert(
            data, kwargs["to_html"], kwargs["to_pdf"], verbose
        )

    # if user enter only a date
    elif date is not None and source is None:
        json_flag, limit = kwargs["json"], kwargs["limit"]
        st_manager = FindManagerWhenEnterDate(
            source, date=date, verbose=verbose, json_flag=json_flag,
            limit=limit, colorize=kwargs["colorize"]
        )
        paths = st_manager.check_news_by_date()

        if not paths:
            return False

        list_of_content = st_manager.get_content_by_paths(paths)
        if verbose:
            info_print(
                "The news has been successfully extracted from the storage"
            )
        list_of_content = st_manager.slice_content_by_limit(list_of_content)
        if not list_of_content:
            return False
        st_manager.data_output(list_of_content)

        interface_to_convert(
            list_of_content, kwargs["to_html"], kwargs["to_pdf"], verbose
        )

    # if user enter a date and a source
    elif date is not None and source is not None:
        json_flag, limit = kwargs["json"], kwargs["limit"]
        st_manager = FindManagerWhenEnterDateAndSource(
            source, date=date, verbose=verbose, json_flag=json_flag,
            limit=limit, colorize=kwargs["colorize"]
        )
        paths = st_manager.check_news_by_date()
        st_manager.date = st_manager.get_date_in_correct_format(date)

        if not paths:
            return False
        file_name = st_manager.get_file_name()

        path = list(filter(lambda x: x[-len(file_name):] == file_name, paths))
        if not path:
            st_manager.news_was_not_founded()
            return False

        data = st_manager.read_from_storage(path[0])
        if verbose:
            info_print(
                "The news has been successfully extracted from the storage"
            )

        data = st_manager.slice_content_by_limit(data)
        if not data:
            return False
        st_manager.data_output(data)

        interface_to_convert(
            data, kwargs["to_html"], kwargs["to_pdf"], verbose
        )



from cool_project.cervices.print_functions import (
    info_print, warning_print, error_print
)


def test_info_print(capsys):
    """test for info_print"""
    info_print("Message")
    captured = capsys.readouterr()
    assert captured.out == "[INFO] Message\n\n"


def test_warning_print(capsys):
    """test for warning_print"""
    warning_print("Message")
    captured = capsys.readouterr()
    assert captured.out == "[WARNING] Message\n\n"


def test_error_print(capsys):
    """test for error_print"""
    error_print("Message")
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] Message\n\n"
    assert captured.out != "[ERROR] Message\n"



from pathlib import Path

from jinja2 import Environment, select_autoescape, FileSystemLoader

from cool_project.cervices.print_functions import error_print, info_print
from project_settings import FILE_NAME_HTML


def make_dir(path):
    """
    Creating a folder at the got path.
    If the folder already exists does nothing.

    :param path: the path where the folder should be created
    """
    if not Path(path).exists():
        p = Path(path)
        p.mkdir(parents=True)


def is_list(obj):
    """
    Check obj is list.

    :param obj: object
    :return: True or False
    """
    return isinstance(obj, list)


def get_env():
    """
    Init Environment for jinja2
    :return: env
    """
    path = Path(__file__).parent
    path = Path(path, "templates")
    env = Environment(
        loader=FileSystemLoader(path),
        autoescape=select_autoescape(["html"]),
        trim_blocks=True,
        lstrip_blocks=True,
    )
    return env


def get_content(data, env):
    """
    Creating a dictionary with content and a
    dictionary with tests for env.

    :param data: news
    :param env: env
    :return: content
    """
    if not is_list(data[0]):
        data = [data]
    content = {
        "title": "Feeds",
        "feeds": data
    }
    env.tests["is_list"] = is_list
    return content


def convert_to_html(data, path, verbose):
    """
    Еhe function gets a list of dictionaries with
    news, a path, and a verbose flag.
    Initializes the Environment of jinja2, gets the HTML template,
    runs the render, and saves the file to the received path.

    :param data: a list of dictionaries with news
    :param path: the path to save the file
    :param verbose: verbose mode
    """
    if verbose:
        info_print("Conversion to HTML started")

    env = get_env()
    content = get_content(data, env)
    template = env.get_template("template.html")
    result = template.render(content)
    path = Path(path)
    if verbose:
        info_print("Conversion to HTML ended")
    try:
        make_dir(path)
        with open(Path(path, FILE_NAME_HTML), "w", encoding="utf-8") as file:
            file.write(result)
        info_print(
            f"A feed in HTML format was saved on the path: "
            f"{Path(path, FILE_NAME_HTML)}"
        )
    except Exception:
        error_print("The entered path cannot be created")



from requests import exceptions
import shutil
from math import ceil

from cool_project.cervices.print_functions import (
    info_print, warning_print, error_print
)


def check_limit_type_value(func):
    """
    Decorator which check type of limit value.
    """

    def wrapper(*args, **kwargs):
        result = func(*args, **kwargs)
        try:
            if result.limit is not None:
                result.limit = int(result.limit)
        except ValueError:
            warning_print("You must enter the number in --limit params.")
        return result

    return wrapper


def intercept_errors(func):
    """Decorator which intercept a errors"""

    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except exceptions.ConnectionError:
            error_print("Connection error. Please check your URL")
        except exceptions.MissingSchema as exc:
            error_print(exc)
        except Exception as exc:
            error_print("Unknown error")

    return wrapper


def verbose_information_about_start_scrapping(func):
    """
    Decorator which print information about start and end scrapping
    """

    def wrapper(*args, **kwargs):
        if args[0].verbose:
            info_print("Start Scrapping")
            result = func(*args, **kwargs)
            info_print("Stop Scrapping")
        else:
            result = func(*args, **kwargs)

        return result

    return wrapper


def get_data_for_print_delimiter(word):
    """
    Get data for print separator for function

    :param word: a word which will be printed in center of a separator
    :return: left_columns_count, right_columns_count, word
    """
    columns = shutil.get_terminal_size().columns

    if word is None:
        word = ""
    else:
        word = f" {word} "

    columns_count = columns - len(word)
    left_columns_count = ceil(columns_count / 2)
    right_columns_count = columns_count - left_columns_count

    return left_columns_count, right_columns_count, word


def decorator_delimiter(start_word=None, end_word=None, calls_stat=False):
    """
    Decorator which print separate line with
    start_word and end_word value,
    if calls_stat is True also print how much times function was called,
    if start_word or end_word value is None value will not be print.

    :param start_word: a word which will be printed
    in center of a start separator
    :param end_word: a word which will be printed
    in center of a end separator
    :param calls_stat: print or not how much times function was called
    """
    count_of_calls = 1

    def decorator(func):

        def wrapper(*args, **kwargs):
            nonlocal count_of_calls, start_word
            word = start_word

            if calls_stat:
                if start_word is not None:
                    word = f"{start_word} {count_of_calls}"
                    count_of_calls += 1

            left_columns_count, right_columns_count, word = get_data_for_print_delimiter(word)
            print(
                f"{'-' * left_columns_count}{word}{'-' * right_columns_count}"
            )
            result = func(*args, **kwargs)
            left_columns_count, right_columns_count, word = get_data_for_print_delimiter(end_word)
            print(
                f"{'-' * left_columns_count}{word}{'-' * right_columns_count}"
            )

            return result

        return wrapper

    return decorator



from pathlib import Path

from fpdf import FPDF

from cool_project.cervices.print_functions import info_print, error_print
from project_settings import FILE_NAME_PDF


class PDF(FPDF):
    """
    Class that generates the PDF file
    """

    def _get_item(self, news):
        """
        the method that generates the news.

        :param news: dict
        """
        if news["title"]:
            self.multi_cell(0, 5, f"Title: {news['title']}")
            self.ln()
        if news["date"]:
            self.multi_cell(0, 5, f"date of publication: {news['date']}")
            self.ln()
        if news["link"]:
            self.multi_cell(0, 5, f"Link: {news['link']}")
            self.ln()
        if news["author"]:
            self.multi_cell(0, 5, f"Author: {news['author']}")
            self.ln()

        if news["category"]:
            if self.is_list(news["category"]):
                for category in news["category"]:
                    self.cell(0, 5, "Categories: ", ln=1)
                    self.multi_cell(0, 5, f"{' ' * 5}{category}")
            else:
                self.multi_cell(0, 5, f"Category: {news['category']}")
            self.ln()

        if news["description"]:
            self.multi_cell(0, 5, f"Description: {news['description']}")
            self.ln()
        if news["more_description"]:
            self.multi_cell(
                0, 5, f"More description: {news['more_description']}"
            )
            self.ln()
        if news["comments"]:
            self.multi_cell(0, 5, f"Comments: {news['comments']}")
            self.ln()

        if news["media_object"]:
            if self.is_list(news["media_object"]):
                for media in news["media_object"]:
                    self.cell(0, 5, "Media objects: ", ln=1)
                    self.multi_cell(0, 5, f"{' ' * 5}{media}")
            else:
                self.multi_cell(0, 5, f"Media object: {news['media_object']}")
            self.ln()

        if news["extra_links"]:
            if self.is_list(news["extra_links"]):
                for extra_link in news["extra_links"]:
                    self.cell(0, 5, "Extra links: ", ln=1)
                    self.multi_cell(0, 5, f"{' ' * 5}{extra_link}")
            else:
                self.multi_cell(0, 5, f"Extra link: {news['extra_links']}")
            self.ln()

        if news["source_feed"]:
            if self.is_list(news["source_feed"]):
                for source in news["source_feed"]:
                    self.cell(0, 5, "Sources: ", ln=1)
                    self.multi_cell(0, 5, f"{' ' * 5}{source}")
            else:
                self.multi_cell(0, 5, f"Source: {news['source_feed']}")

    def body(self, data):
        """
        the method that generates the feeds
        :param data: a list of dictionaries with news
        """
        if not self.is_list(data[0]):
            data = [data]

        for feed in data:
            self.cell(
                0, 5, f'Channel title: {feed[0]["channel_title"]}',
                align="C", ln=1
            )
            self.cell(0, 5, f'Source: {feed[0]["source"]}', align="C", ln=1)

            for news in feed[1:]:
                self.cell(0, 5, f"{'-' * 125}", align="C", ln=1)

                self._get_item(news)

                self.cell(0, 5, f"{'-' * 125}", align="C")
                self.ln()
                self.ln()

    def footer(self):
        """
        footer generation
        """
        self.set_y(-10)
        self.set_font("DejaVu", "", 15)
        self.cell(0, 5, f"Page {str(self.page_no())}", 0, 0, "C")

    @staticmethod
    def is_list(obj):
        """
        Check obj is list.

        :param obj: object
        :return: True or False
        """
        return isinstance(obj, list)

    @staticmethod
    def make_dir(path):
        """
        Creating a folder at the got path.
        If the folder already exists does nothing.

        :param path: the path where the folder should be created
        """
        if not Path(path).exists():
            p = Path(path)
            p.mkdir(parents=True)


def convertor_to_pdf(data, path, verbose):
    """
    Еhe function gets a list of dictionaries with
    news, a path, and a verbose flag.
    the function gets the path to TTF with fonts and
    sets the resulting fonts to the PDF class,
    generates PDF from the received data.
    Saves the file to the received path.

    :param data: a list of dictionaries with news
    :param path: the path to save the file
    :param verbose: verbose mode
    """
    pdf = PDF()
    pdf.alias_nb_pages()
    pdf.add_page()
    path_to_ttf = Path(
        Path(__file__).parent, "files_for_pdf", "DejaVuSansCondensed.ttf"
    )
    pdf.add_font("DejaVu", "", path_to_ttf, uni=True)

    if verbose:
        info_print("Fonts have been received")

    pdf.set_font("DejaVu", "", 14)
    pdf.set_auto_page_break(True, 10)

    if verbose:
        info_print("PDF generation started")
    pdf.body(data)
    if verbose:
        info_print("PDF has been generated")

    try:
        pdf.make_dir(path)
        path = Path(path, FILE_NAME_PDF)
        pdf.output(path, "F")
        info_print(f"A feed in PDF format was saved on the path: {path}")
    except PermissionError:
        error_print(
            f"it is not possible to save a PDF file on this path: {path}."
            f" Such file already exists or cannot access the folder."
        )
    except Exception:
        error_print("The entered path cannot be created")



import shutil
from math import ceil
from pathlib import Path

import pytest

from cool_project.data_storage.storage_managers import (
    StorageManager as SM,
    DataManagerInStorageAfterParsing,
    FindManagerWhenEnterDate,
    FindManagerWhenEnterDateAndSource
)


@pytest.fixture()
def init_SM():
    """init StorageManager"""
    data = ["data"]
    st_manager = SM("sour.ce", verbose=True, data=data, colorize=False)
    return st_manager


@pytest.fixture()
def init_DataManagerInStorageAfterParsing():
    """init DataManagerInStorageAfterParsing"""
    data = [
        {
            "channel_title": "Люди Onlíner",
            "source": "source"
        },
        {
            "title": "Пиневич: 40% коечного фонда "
                     "перепрофилировано под ковидных пациентов",
            "date": "Sat, 23 Oct 2021 17:51:11 +0300",
            "link": "https://people.onliner.by/2021/10/23/"
                    "pereprofilirovano-pod-kovidnyx-pacientov",
            "author": None,
            "category": ["Здоровье"],
            "description": 'По всей стране в медицинских учреждениях '
                           'перепрофилировано чуть более 40% коечного фонда. '
                           'Такую цифру привел глава Министерства '
                           'здравоохранения Дмитрий Пиневич, '
                           'отметив, что это позволяет и '
                           'дальше оказывать плановую помощь '
                           'жителям Беларуси.Читать далее…',
            "more_description": None,
            "comments": None,
            "media_object": ["https://content.onliner.by/news/thumbnail/"
                             "5a7aa9c81d307b0ddc03a0f10746bffe.jpeg"],
            "extra_links": "https://people.onliner.by/2021/10/23/"
                           "pereprofilirovano-pod-kovidnyx-pacientov",
            "source_feed": None,
        }
    ]
    st_manager = DataManagerInStorageAfterParsing(
        "sour.ce", verbose=True, data=data, colorize=False
    )
    return st_manager


@pytest.fixture()
def init_FindManagerWhenEnterDateAndSource():
    """init FindManagerWhenEnterDateAndSource"""
    st_manager = FindManagerWhenEnterDateAndSource(
        "sour.ce", verbose=True, date="1000-10-23",
        json_flag=True, limit=1, colorize=False
    )
    return st_manager


@pytest.fixture()
def init_FindManagerWhenEnterDate():
    """init FindManagerWhenEnterDate"""
    st_manager = FindManagerWhenEnterDate(
        "sour.ce", verbose=True, date="1000-10-23",
        json_flag=True, limit=1, colorize=False
    )
    return st_manager


@pytest.fixture()
def del_dir():
    """A fixture that deletes a directory"""
    path = Path(Path(__file__).parent, "test_dir")
    yield path
    Path.rmdir(path)


@pytest.fixture()
def path_to_storage():
    """A fixture that return path to storage"""
    path = Path(__file__).parent.parent.parent
    path = Path(path, "cool_project", "data_storage", "storage")
    return path


@pytest.fixture()
def del_file_txt():
    """A fixture that deletes file"""
    path = Path(Path(__file__).parent, "feed.txt")
    yield
    Path.unlink(path)


def test_StorageManager_method_get_abspath_to_storage(
        init_SM, path_to_storage
):
    """test for StorageManager method get_abspath_to_storage"""
    st_manger = init_SM
    path = st_manger._get_abspath_to_storage()
    assert path == path_to_storage


def test_StoragerManager_method_write_to_storage(del_file_txt, init_SM):
    """test for StoragerManager method write_to_storage"""
    path = Path(Path(__file__).parent, "feed.txt")
    st_manager = init_SM
    st_manager.write_to_storage(path, 'data')
    assert Path(path).exists()


def test_StorageManager_method_read_from_storage(init_SM):
    """test for StorageManager method read_from_storage"""
    path = Path(Path(__file__).parent, "test_data.json")
    st_manager = init_SM
    result = st_manager.read_from_storage(path)
    assert result == [{"test": "data"}]


def test_StorageManager_method_make_dir(init_SM, del_dir):
    """test for StorageManager method make_dir"""
    path = del_dir
    st_manager = init_SM
    st_manager.make_dir(path)
    assert Path(path).exists()


def test_StorageManager_method_path_is_exists(init_SM):
    """test for StorageManager method path_is_exists"""
    path = Path(__file__)
    st_manager = init_SM
    assert st_manager.path_is_exists(path)

    path = Path(Path(__file__), "fake")
    assert not st_manager.path_is_exists(path)


def test_StorageManager_method_get_path(init_SM):
    """test for StorageManager method get_path"""
    expected = Path(Path(__file__), "check")
    st_manager = init_SM
    actual = st_manager.get_path(Path(__file__), "check")
    assert actual == expected


@pytest.mark.parametrize("date_str, correct_res",
                         [("Sat, 23 Oct 2021 17:51:11 +0300", "2021-10-23"),
                          ("Sat, 23 Oct 2021 some inf", None),
                          ("20211024", "2021-10-24"),
                          ("22024", None)]
                         )
def test_StorageManager_method_get_date_in_correct_format(
        init_SM, date_str, correct_res
):
    """test for StorageManager method get_date_in_correct_format"""
    st_manager = init_SM
    actual = st_manager.get_date_in_correct_format(date_str)
    assert actual == correct_res


def test_DataManagerInStorageAfterParsing_method_get_file_name(
        init_DataManagerInStorageAfterParsing
):
    """test for DataManagerInStorageAfterParsing method get_file_name"""
    st_manager = init_DataManagerInStorageAfterParsing
    actual = st_manager.get_file_name("2021-10-22")
    expected = "2021-10-22_sour_ce.json"
    assert actual == expected


def test_DataManagerInStorageAfterParsing_method_make_dir_by_key(
        init_DataManagerInStorageAfterParsing, path_to_storage
):
    """test for DataManagerInStorageAfterParsing method make_dir_by_key"""
    st_manager = init_DataManagerInStorageAfterParsing
    data = {"1000-10-23": "data"}
    expected = Path(path_to_storage, "1000-10", "1000-10-23")
    st_manager.make_dir_by_key(data)
    assert Path(expected).exists()
    shutil.rmtree(Path(path_to_storage, "1000-10"))


def test_DataManagerInStorageAfterParsing_method_control_of_exist(
        init_DataManagerInStorageAfterParsing, path_to_storage
):
    """test for DataManagerInStorageAfterParsing method control_of_exist"""
    st_manager = init_DataManagerInStorageAfterParsing
    data = {"1000-10-23": [{"data": "data"}]}
    channel_data = [{"channel_data": "data"}]
    st_manager.make_dir_by_key(data)
    st_manager.control_of_exist(data, channel_data)
    expected = Path(path_to_storage, "1000-10", "1000-10-23")
    assert Path(expected).exists()

    data = {"1000-10-23": [{"data2": "data2"}]}
    st_manager.control_of_exist(data, channel_data)
    assert Path(expected).exists()
    shutil.rmtree(Path(path_to_storage, "1000-10"))


def test_DataManagerInStorageAfterParsing_method_split_data_by_news(
        init_DataManagerInStorageAfterParsing, capsys
):
    """test for DataManagerInStorageAfterParsing method split_data_by_news"""
    st_manager = init_DataManagerInStorageAfterParsing
    channel_data, dict_for_data_saving = st_manager.split_data_by_news()
    channel_data_correct = [{
        "channel_title": "Люди Onlíner",
        "source": "source"
    }]
    dict_for_data_saving_correct = {
        "2021-10-23": [
            {
                "title": "Пиневич: 40% коечного фонда перепрофилировано"
                         " под ковидных пациентов",
                "date": "Sat, 23 Oct 2021 17:51:11 +0300",
                "link": "https://people.onliner.by/2021/10/23/"
                        "pereprofilirovano-pod-kovidnyx-pacientov",
                "author": None,
                "category": ["Здоровье"],
                "description": 'По всей стране в медицинских учреждениях '
                               'перепрофилировано чуть более 40% '
                               'коечного фонда. Такую цифру '
                               'привел глава Министерства '
                               'здравоохранения Дмитрий Пиневич, '
                               'отметив, что это позволяет и '
                               'дальше оказывать плановую помощь '
                               'жителям Беларуси.Читать далее…',
                "more_description": None,
                "comments": None,
                "media_object": ["https://content.onliner.by/news/thumbnail/"
                                 "5a7aa9c81d307b0ddc03a0f10746bffe.jpeg"],
                "extra_links": "https://people.onliner.by/2021/10/23/"
                               "pereprofilirovano-pod-kovidnyx-pacientov",
                "source_feed": None,
            }
        ]
    }
    captured = capsys.readouterr()
    assert (channel_data, dict_for_data_saving) == (
        channel_data_correct, dict_for_data_saving_correct
    )
    assert captured.out == ""

    data = [
        {
            "channel_title": "Люди Onlíner",
            "source": "source"
        },
        {
            "title": "Пиневич: 40% коечного фонда перепрофилировано "
                     "под ковидных пациентов",
            "date": "Sat, 23 Oct 021 17:51:11 +0300",
            "link": "https://people.onliner.by/2021/10/23/"
                    "pereprofilirovano-pod-kovidnyx-pacientov",
            "author": None,
        }
    ]
    st_manager = DataManagerInStorageAfterParsing(
        "sour.ce", verbose=True, data=data, colorize=False
    )
    actual = st_manager.split_data_by_news()
    captured = capsys.readouterr()

    # captured.out[82:] - supported dates. Supported dates may change
    assert captured.out[:82] == f"[ERROR] The site sour.ce uses an" \
                                f" unsupported date format. " \
                                f"Storage data has failed."
    assert actual is None


def test_FindManagerWhenEnterDateAndSource_method_get_file_name(
        init_FindManagerWhenEnterDateAndSource
):
    """test for FindManagerWhenEnterDateAndSource method get_file_name"""
    st_manager = init_FindManagerWhenEnterDateAndSource
    file_name = st_manager.get_file_name()
    assert file_name == "1000-10-23_sour_ce.json"


def test_FindManagerWhenEnterDateAndSource_method_news_was_not_founded(
        init_FindManagerWhenEnterDateAndSource, capsys
):
    """
    test for FindManagerWhenEnterDateAndSource method news_was_not_founded
    """
    st_manager = init_FindManagerWhenEnterDateAndSource
    st_manager.news_was_not_founded()
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] No news was founded " \
                           "for this date: 1000-10-23," \
                           " and this source: sour.ce\n\n"


def calculate_terminal_size(word):
    """
    calculated terminal size
    :param word: a word which will be printed in center of a separator
    :return: left_columns_count, right_columns_count, word
    """
    columns = shutil.get_terminal_size().columns

    if word is None:
        word = ""
    else:
        word = f" {word} "

    columns_count = columns - len(word)
    left_columns_count = ceil(columns_count / 2)
    right_columns_count = columns_count - left_columns_count
    return left_columns_count, right_columns_count, word


def test_FindManagerWhenEnterDateAndSource_method_data_output(
        init_FindManagerWhenEnterDateAndSource, capsys
):
    """test for FindManagerWhenEnterDateAndSource method data_output"""
    st_manager = init_FindManagerWhenEnterDateAndSource
    data = [{"data": "LOL"}]
    st_manager.data_output(data)
    captured = capsys.readouterr()
    assert captured.out == "[INFO] News will be printed in JSON format\n\n" \
                           "[\n" \
                           "    {\n" \
                           "        \"data\": \"LOL\"\n" \
                           "    }\n" \
                           "]\n"

    st_manager = FindManagerWhenEnterDateAndSource(
        "sour.ce", verbose=True, date="1000-10-23",
        json_flag=None, limit=1, colorize=False
    )
    data = [
        {
            "channel_title": "channel_title",
            "source": "source"
        },
        {
            "title": "title",
            "date": "date",
            "link": "link",
            "author": None,
            "category": ["Здоровье"],
            "description": 'description',
            "more_description": None,
            "comments": None,
            "media_object": "media_object",
            "extra_links": "extra_links",
            "source_feed": None,
        }
    ]
    st_manager.data_output(data)
    captured = capsys.readouterr()
    first_line_1 = calculate_terminal_size("News 1")
    last_line = calculate_terminal_size(None)
    assert captured.out == f"[INFO] News will be printed " \
                           f"in a standard format\n\n" \
                           f"\n\nFeed source: source\n" \
                           f"Feed: channel_title\n\n" \
                           f"{'-' * first_line_1[0]}{first_line_1[2]}" \
                           f"{'-' * first_line_1[1]}\n" \
                           f"Title: title\n" \
                           f"Date: date\n" \
                           f"Link: link\n" \
                           f"Category: Здоровье\n\n" \
                           f"Description: description\n\n" \
                           f"Media Object: media_object\n" \
                           f"Extra Links: extra_links\n" \
                           f"{'-' * last_line[0]}{last_line[2]}" \
                           f"{'-' * last_line[1]}\n\n"


def test_FindManagerWhenEnterDateAndSource_method_slice_content_by_limit(
        init_FindManagerWhenEnterDateAndSource, capsys
):
    """
    test for FindManagerWhenEnterDateAndSource method slice_content_by_limit
    """
    st_manager = init_FindManagerWhenEnterDateAndSource
    data = [
        {"channel_title": "Люди Onlíner"},
        {"2021-10-23": {"title": "title"}},
        {"2021-10-23": {"title": "title"}}
    ]
    actual = st_manager.slice_content_by_limit(data)
    capsys.readouterr()
    assert actual == data[:-1]

    st_manager = FindManagerWhenEnterDateAndSource(
        "sour.ce", verbose=True, date="1000-10-23",
        json_flag=True, limit=None, colorize=False
    )
    actual = st_manager.slice_content_by_limit(data)
    capsys.readouterr()
    assert actual == data

    st_manager = FindManagerWhenEnterDateAndSource(
        "sour.ce", verbose=True, date="1000-10-23",
        json_flag=True, limit=-3, colorize=False
    )
    st_manager.slice_content_by_limit(data)
    captured = capsys.readouterr()
    assert captured.out == "[INFO] The news was searched in the " \
                           "storage by date: 1000-10-23, " \
                           "and source: sour.ce\n\n" \
                           "[ERROR] The limit is less than or equal" \
                           " to 0, news cannot be printed.\n\n"


def test_FindManagerWhenEnterDate_method_get_content_by_paths(
        init_FindManagerWhenEnterDate
):
    """test for FindManagerWhenEnterDate method get_content_by_paths"""
    st_manager = init_FindManagerWhenEnterDate
    path = Path(Path(__file__).parent, "test_data.json")
    list_of_content = st_manager.get_content_by_paths([path])
    assert list_of_content == [[{'test': 'data'}]]


def test_FindManagerWhenEnterDate_method_slice_content_by_limit(
        init_FindManagerWhenEnterDate, capsys
):
    """test for FindManagerWhenEnterDate method slice_content_by_limit"""
    st_manager = init_FindManagerWhenEnterDate
    data = [
        {"source": "Люди Onlíner"},
        {"2021-10-23": {"title": "title"}},
        {"2021-10-23": {"title": "title"}}
    ]
    actual = st_manager.slice_content_by_limit([data])
    capsys.readouterr()
    assert actual == [data[:-1]]

    st_manager = FindManagerWhenEnterDate(
        "sour.ce", verbose=True, date="1000-10-23",
        json_flag=True, limit=None, colorize=False
    )
    actual = st_manager.slice_content_by_limit([data])
    capsys.readouterr()
    assert actual == [data]

    st_manager = FindManagerWhenEnterDate(
        "sour.ce", verbose=True, date="1000-10-23",
        json_flag=True, limit=3, colorize=False
    )
    actual = st_manager.slice_content_by_limit([data])
    capsys.readouterr()
    assert actual == [data]

    st_manager = FindManagerWhenEnterDate(
        "sour.ce", verbose=True, date="1000-10-23",
        json_flag=True, limit=-1, colorize=False
    )
    st_manager.slice_content_by_limit([data])
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] The limit is less than or " \
                           "equal to 0, news cannot be printed.\n\n"


def test_FindManagerWhenEnterDate_method_news_was_not_founded(
        init_FindManagerWhenEnterDate, capsys
):
    """test for FindManagerWhenEnterDate method news_was_not_founded"""
    st_manager = init_FindManagerWhenEnterDate
    st_manager.news_was_not_founded("1000-10-23")
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] No news was founded for this date:" \
                           " 1000-10-23, and this source: sour.ce\n\n"


def test_FindManagerWhenEnterDate_method_data_output(
        init_FindManagerWhenEnterDate, capsys
):
    """test for FindManagerWhenEnterDate method data_output"""
    st_manager = init_FindManagerWhenEnterDate
    data = [{"data": "LOL"}]
    st_manager.data_output(data)
    captured = capsys.readouterr()
    assert captured.out == "[INFO] News will be printed in " \
                           "JSON format\n\n" \
                           "[\n" \
                           "    {\n" \
                           "        \"data\": \"LOL\"\n" \
                           "    }\n" \
                           "]\n"

    st_manager = FindManagerWhenEnterDate(
        "sour.ce", verbose=True, date="1000-10-23",
        json_flag=None, limit=1, colorize=False
    )
    data = [
        {
            "channel_title": "channel_title",
            "source": "source"
        },
        {
            "title": "title",
            "date": "date",
            "link": "link",
            "author": None,
            "category": ["Здоровье"],
            "description": 'description',
            "more_description": None,
            "comments": None,
            "media_object": "media_object",
            "extra_links": "extra_links",
            "source_feed": None,
        }
    ]
    st_manager.data_output([data])
    captured = capsys.readouterr()
    first_line_1 = calculate_terminal_size("News 2")
    last_line = calculate_terminal_size(None)
    assert captured.out == f"[INFO] News will be printed " \
                           f"in a standard format\n\n" \
                           f"\n\nFeed source: source\n" \
                           f"Feed: channel_title\n\n" \
                           f"{'-' * first_line_1[0]}{first_line_1[2]}" \
                           f"{'-' * first_line_1[1]}\n" \
                           f"Title: title\n" \
                           f"Date: date\n" \
                           f"Link: link\n" \
                           f"Category: Здоровье\n\n" \
                           f"Description: description\n\n" \
                           f"Media Object: media_object\n" \
                           f"Extra Links: extra_links\n" \
                           f"{'-' * last_line[0]}{last_line[2]}" \
                           f"{'-' * last_line[1]}\n\n"


def test_FindManagerWhenEnterDate_method_check_news_by_date(
        init_FindManagerWhenEnterDate, capsys
):
    """test for FindManagerWhenEnterDate method check_news_by_date"""
    st_manager = init_FindManagerWhenEnterDate
    st_manager.check_news_by_date()
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] '1000-10-23' is an incorrect date. " \
                           "Please try to enter the date " \
                           "in a correct format\n\n"

    st_manager = FindManagerWhenEnterDate(
        None, verbose=True, date="10001023",
        json_flag=True, limit=1, colorize=False
    )
    st_manager.check_news_by_date()
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] No news was found for this date" \
                           " - 1000-10-23\n\n"

    path = st_manager._get_abspath_to_storage()
    path = Path(path, "1000-10")
    st_manager.make_dir(path)

    st_manager.check_news_by_date()
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] No news was found for this date" \
                           " - 1000-10-23\n\n"

    path = Path(path, "1000-10-23")
    st_manager.make_dir(path)

    res = st_manager.check_news_by_date()
    assert res == []

    shutil.rmtree(Path(st_manager._get_abspath_to_storage(), "1000-10"))


import shutil
from math import ceil

import pytest

from cool_project.cervices.data_output import (
    console_output_feed,
    output_feed,
    line_break,
    rebuild_value,
    console_json_output,
)

DATA = [
    {
        'channel_title': 'Lenta.ru : Новости',
        'source': 'https://lenta.ru/rss'
    },
    {
        'title': '«Краснодар» разгромил новичка '
                 'РПЛ и вышел на третье место',
        'date': 'Sat, 23 Oct 2021 15:51:57 +0300',
        'link': 'https://lenta.ru/news/2021/10/23/tretyemesto/',
        'author': 'Алексей Гусев',
        'category': ['Спорт'],
        'description': '«Краснодар» на выезде обыграл «Нижний Новгород»'
                       ' в матче 12-го тура Российской премьер-лиги'
                       ' (РПЛ). Встреча прошла в субботу, 23 октября, '
                       'и завершилась со счетом 4:1 в пользу гостей. '
                       'Одержать разгромную победу краснодарцам '
                       'позволили голы Гжегожа Крыховяка, '
                       'Эдуарда Сперцяна, '
                       'Александра Черникова и Владимира Ильина.',
        'more_description': None,
        'comments': None,
        'media_object': 'https://icdn.lenta.ru/images/2021/10/'
                        '23/15/20211023154554818/'
                        'pic_fbc5daebf5e79eab60893757b5d7f532.jpg',
        'extra_links': 'https://lenta.ru/news/2021/10/23/tretyemesto/',
        'source_feed': None
    },
    {
        'title': 'Потомков западных интервентов '
                 'в России призвали изучать историю',
        'date': 'Sat, 23 Oct 2021 15:39:00 +0300',
        'link': 'https://lenta.ru/news/2021/10/23/naryshkin_/',
        'author': 'Варвара Кошечкина',
        'category': ['Россия'],
        'description': 'Западные страны, которые в прошлом '
                       'планировали захватить Россию, '
                       'должны как следует изучать историю, '
                       'чтобы смириться с невыполнимостью '
                       'подобных надежд. С таким заявлением '
                       'выступил глава Службы внешней разведки'
                       ' России Сергей Нарышкин в ходе '
                       'церемонии награждения победителей '
                       'Всероссийского конкурса краеведов.',
        'more_description': None,
        'comments': None,
        'media_object': 'https://icdn.lenta.ru/images/2021/'
                        '10/23/15/20211023151836599/'
                        'pic_ec551103d6e4f2e3642e3ba58ac8ca1b.jpg',
        'extra_links': 'https://lenta.ru/news/2021/10/23/naryshkin_/',
        'source_feed': None
    }
]


def calculate_terminal_size(word):
    """
    calculated terminal size
    :param word: a word which will be printed in center of a separator
    :return: left_columns_count, right_columns_count, word
    """
    columns = shutil.get_terminal_size().columns

    if word is None:
        word = ""
    else:
        word = f" {word} "

    columns_count = columns - len(word)
    left_columns_count = ceil(columns_count / 2)
    right_columns_count = columns_count - left_columns_count
    return left_columns_count, right_columns_count, word


def test_console_output_feed(capsys):
    """test for console_output_feed function"""
    console_output_feed(DATA, False)
    captured = capsys.readouterr()

    first_line_1 = calculate_terminal_size("News 3")
    first_line_2 = calculate_terminal_size("News 4")
    last_line = calculate_terminal_size(None)

    output = f'\n\nFeed source: {DATA[0]["source"]}\n' \
             f'Feed: {DATA[0]["channel_title"]}\n\n' \
             f'{"-" * first_line_1[0]}{first_line_1[2]}' \
             f'{"-" * first_line_1[1]}\n' \
             f'Title: {DATA[1]["title"]}\n' \
             f'Date: {DATA[1]["date"]}\n' \
             f'Link: {DATA[1]["link"]}\n' \
             f'Author: {DATA[1]["author"]}\n' \
             f'Category: {DATA[1]["category"][0]}\n\n' \
             f'Description: {DATA[1]["description"]}\n\n' \
             f'Media Object: {DATA[1]["media_object"]}\n' \
             f'Extra Links: {DATA[1]["extra_links"]}\n' \
             f'{"-" * last_line[0]}{last_line[2]}' \
             f'{"-" * last_line[1]}\n\n' \
             f'{"-" * first_line_2[0]}{first_line_2[2]}' \
             f'{"-" * first_line_2[1]}\n' \
             f'Title: {DATA[2]["title"]}\n' \
             f'Date: {DATA[2]["date"]}\n' \
             f'Link: {DATA[2]["link"]}\n' \
             f'Author: {DATA[2]["author"]}\n' \
             f'Category: {DATA[2]["category"][0]}\n\n' \
             f'Description: {DATA[2]["description"]}\n\n' \
             f'Media Object: {DATA[2]["media_object"]}\n' \
             f'Extra Links: {DATA[2]["extra_links"]}\n' \
             f'{"-" * last_line[0]}{last_line[2]}' \
             f'{"-" * last_line[1]}\n\n' \

    assert captured.out == output


@pytest.mark.parametrize("news, colorize",
                         [("News 5", False),]
                         )
def test_output_feed(capsys, news, colorize):
    """test for output_feed function"""
    output_feed(DATA[1], colorize)
    captured = capsys.readouterr()

    first_line_1 = calculate_terminal_size(news)
    last_line = calculate_terminal_size(None)
    output = f'{"-" * first_line_1[0]}{first_line_1[2]}' \
             f'{"-" * first_line_1[1]}\n' \
             f'Title: {DATA[1]["title"]}\n' \
             f'Date: {DATA[1]["date"]}\n' \
             f'Link: {DATA[1]["link"]}\n' \
             f'Author: {DATA[1]["author"]}\n' \
             f'Category: {DATA[1]["category"][0]}\n\n' \
             f'Description: {DATA[1]["description"]}\n\n' \
             f'Media Object: {DATA[1]["media_object"]}\n' \
             f'Extra Links: {DATA[1]["extra_links"]}\n' \
             f'{"-" * last_line[0]}{last_line[2]}' \
             f'{"-" * last_line[1]}\n'

    assert captured.out == output


@pytest.mark.parametrize("word, res",
                         [("KEK", ""),
                          ("description", "\n"),
                          ("comments", "\n")]
                         )
def test_line_break(capsys, word, res):
    """test for line_break function"""
    line_break(word)
    captured = capsys.readouterr()
    assert captured.out == res


@pytest.mark.parametrize("ls, res",
                         [(["How", "I", "Met", "Your", "Mother"],
                           "How, I, Met, Your, Mother"),
                          (["20", "12"], "20, 12")]
                         )
def test_rebuild_value(ls, res):
    """How I Met Your Mother is an American sitcom -
    https://en.wikipedia.org/wiki/How_I_Met_Your_Mother"""
    result = rebuild_value(ls)
    assert result == res


def test_console_json_output(capsys):
    """test for console_json_output function"""
    console_json_output([DATA[0], DATA[1]])
    captured = capsys.readouterr()
    output = "[\n" \
             "    {\n" \
             '        "channel_title": "Lenta.ru : Новости",\n' \
             '        "source": "https://lenta.ru/rss"\n' \
             '    },\n' \
             '    {\n' \
             '        "title": "«Краснодар» разгромил новичка ' \
             'РПЛ и вышел на третье место",\n' \
             '        "date": "Sat, 23 Oct 2021 15:51:57 +0300",\n' \
             '        "link": "https://lenta.ru/news/2021/' \
             '10/23/tretyemesto/",\n' \
             '        "author": "Алексей Гусев",\n' \
             '        "category": [\n' \
             '            "Спорт"\n' \
             '        ],\n' \
             '        "description": ' \
             '"«Краснодар» на выезде обыграл ' \
             '«Нижний Новгород» в матче 12-го тура Российской ' \
             'премьер-лиги (РПЛ). Встреча прошла в субботу, ' \
             '23 октября, и завершилась со счетом 4:1 в ' \
             'пользу гостей. Одержать разгромную победу ' \
             'краснодарцам позволили голы Гжегожа Крыховяка,' \
             ' Эдуарда Сперцяна, Александра Черникова ' \
             'и Владимира Ильина.",\n' \
             '        "more_description": null,\n' \
             '        "comments": null,\n' \
             '        "media_object": ' \
             '"https://icdn.lenta.ru/images' \
             '/2021/10/23/15/20211023154554818/' \
             'pic_fbc5daebf5e79eab60893757b5d7f532.jpg",\n' \
             '        "extra_links": ' \
             '"https://lenta.ru/news/2021/10/23/tretyemesto/",\n' \
             '        "source_feed": null\n' \
             '    }\n' \
             ']\n'

    assert captured.out == output



from pathlib import Path

import pytest

from cool_project.conversion_to_format.conversion_to_html import (
    make_dir,
    is_list,
    get_env,
    get_content,
    convert_to_html
)


@pytest.fixture()
def del_dir():
    """A fixture that deletes a directory"""
    path = Path(Path(__file__).parent, "test_dir")
    yield path
    Path.rmdir(path)


@pytest.fixture()
def del_file_html():
    """A fixture that deletes file"""
    path = Path(Path(__file__).parent, "feed.html")
    yield
    Path.unlink(path)


def test_make_dir(del_dir):
    """test for make_dir"""
    path = del_dir
    make_dir(path)
    assert Path(path).exists()


@pytest.mark.parametrize("obj, correct_res",
                         [("soup_fix", False),
                          ([1, 2], True)]
                         )
def test_is_list(obj, correct_res):
    """test for is_list"""
    assert is_list(obj) == correct_res


def test_get_env():
    """test for get_env"""
    assert get_env()


@pytest.mark.parametrize("data, correct_res",
                         [("data", ["data"]),
                          ([["data"]], [["data"]])]
                         )
def test_get_content(data, correct_res):
    """test for get_content"""
    content = get_content(data, get_env())
    expected = {
        "title": "Feeds",
        "feeds": correct_res
    }
    assert content == expected


def test_convert_to_html(del_file_html, capsys):
    """test for convert_to_htm"""
    data = "data"
    path = Path(__file__).parent
    convert_to_html(data, path, True)
    captured = capsys.readouterr()
    assert captured.out == f"[INFO] Conversion to HTML started\n\n" \
                           f"[INFO] Conversion to HTML ended\n\n" \
                           f"[INFO] A feed in HTML format was saved " \
                           f"on the path: {Path(path, 'feed.html')}\n\n"



from setuptools import setup, find_packages

PROGRAM_VERSION = "5.0.0"

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setup(
    name='rss_readerCLI',
    version=PROGRAM_VERSION,
    author='Aleksey Mikhalkevich',
    author_email='lehado67@gmail.com',
    description="Python reader for RSS feed",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Aleksey-Mikh/Homework/tree/final_task",
    packages=find_packages(),
    py_modules=['rss_reader'],
    python_requires=">=3.9",
    install_requires=[
        'requests',
        'beautifulsoup4',
        'lxml',
        'Jinja2',
        'fpdf',
        'pytest',
        'pytest-cov',
        'colorama',
    ],
    entry_points={
        "console_scripts": [
            "rss_reader = rss_reader:main",
        ],
    }
)



from pathlib import Path

import pytest

from cool_project.conversion_to_format.conversion_to_pdf import (
    PDF,
    convertor_to_pdf,
)


@pytest.fixture()
def del_dir():
    """A fixture that deletes a directory"""
    path = Path(Path(__file__).parent, "test_dir")
    yield path
    Path.rmdir(path)


@pytest.fixture()
def del_file_pdf():
    """A fixture that deletes file"""
    path = Path(Path(__file__).parent, "feed.pdf")
    yield
    Path.unlink(path)


@pytest.fixture()
def init_pdf():
    """A fixture that init pdf"""
    pdf = PDF()
    pdf.alias_nb_pages()
    pdf.add_page()
    pdf.set_auto_page_break(True, 10)
    path_to_ttf = Path(
        Path(__file__).parent.parent.parent,
        "cool_project",
        "conversion_to_format",
        "files_for_pdf",
        "DejaVuSansCondensed.ttf")
    pdf.add_font("DejaVu", "", path_to_ttf, uni=True)
    pdf.set_font("DejaVu", "", 14)
    yield pdf


@pytest.mark.parametrize("obj, correct_res",
                         [("soup_fix", False),
                          ([1, 2], True)]
                         )
def test_PDF_method_is_list(obj, correct_res):
    """test for PDF method is list"""
    pdf = PDF()
    assert pdf.is_list(obj) == correct_res


def test_PDF_method_make_dir(del_dir):
    """test PDF method make dir"""
    path = del_dir
    PDF.make_dir(path)
    assert Path(path).exists()


def test_PDF_method_footer(init_pdf):
    """test for PDF method footer"""
    pdf = init_pdf
    pdf.footer()


def test_PDF_method_body(init_pdf):
    """test PDF method body"""
    pdf = init_pdf
    data = [
        {
            "channel_title": "channel_title",
            "source": "source"
        },
        {
            "title": "title",
            "date": "date",
            "link": "link",
            "author": "author",
            "category": ["Здоровье"],
            "description": 'description',
            "more_description": "more_description",
            "comments": "comments",
            "media_object": ["media_object"],
            "extra_links": ["extra_links"],
            "source_feed": ["source_feed"],
        }
    ]
    pdf.body(data)


def test_PDF_method_get_item(init_pdf):
    """test PDF method _get_item"""
    pdf = init_pdf
    data = {
        "title": "title",
        "date": "date",
        "link": "link",
        "author": "author",
        "category": "Здоровье",
        "description": 'description',
        "more_description": "more_description",
        "comments": "comments",
        "media_object": "media_object",
        "extra_links": "extra_links",
        "source_feed": "source_feed",
    }

    pdf._get_item(data)


def test_convertor_to_pdf(del_file_pdf, capsys):
    """test for convertor_to_pdf"""
    data = [
        {
            "channel_title": "channel_title",
            "source": "source"
        },
        {
            "title": "title",
            "date": "date",
            "link": "link",
            "author": "author",
            "category": ["Здоровье"],
            "description": 'description',
            "more_description": "more_description",
            "comments": "comments",
            "media_object": ["media_object"],
            "extra_links": ["extra_links"],
            "source_feed": ["source_feed"],
        }
    ]

    convertor_to_pdf(data, Path(__file__).parent, True)
    captured = capsys.readouterr()
    assert captured.out == f"[INFO] Fonts have been received\n\n" \
                           f"[INFO] PDF generation started\n\n" \
                           f"[INFO] PDF has been generated\n\n" \
                           f"[INFO] A feed in PDF format was saved " \
                           f"on the path: " \
                           f"{Path(Path(__file__).parent, 'feed.pdf')}\n\n"



import json
import os
import re
from pathlib import Path
import datetime

from cool_project.cervices.print_functions import error_print, info_print
from cool_project.cervices.data_output import (
    console_output_feed, console_json_output
)
from project_settings import LIST_OF_DATE_FORMATS


class StorageManager:
    """
    A base class for working with storage that implements methods
    for writing data to storage, reading from storage,
    getting paths, and converting dates.
    """

    def __init__(self, source, *, verbose, date=None, data=None, colorize):
        """
        Init StorageManager
        :param source: news source
        :param verbose: verbose mode
        :param date: the date on which you need to receive the news
        :param data: data that need to write to the storage
        """
        self.date = date
        self.source = source
        self.data = data
        self.verbose = verbose
        self.colorize = colorize

    def _get_abspath_to_storage(self):
        """
        Getting the absolute path to this file.
        :return: absolute path to the file
        """
        abs_file_path = os.path.abspath(__file__)
        path, name = os.path.split(abs_file_path)
        path = self.get_path(path, "storage")
        return path

    @staticmethod
    def write_to_storage(path, data):
        """
        Writing data to the storage by the received path.
        :param path: the path where the data should be stored
        :param data: storage data
        """
        with open(path, "w", encoding='utf-8') as file:
            json.dump(data, file, indent=4, ensure_ascii=False)

    @staticmethod
    def read_from_storage(path):
        """
        Reading data from storage by the received path.
        :param path: the path by which the data should be received
        """
        with open(path, encoding='utf-8') as File:
            data = json.load(File)
            return data

    @staticmethod
    def make_dir(path):
        """
        Creating a folder at the got path.
        If the folder already exists does nothing.
        :param path: the path where the folder should be created
        """
        if not Path(path).exists():
            p = Path(path)
            p.mkdir()

    @staticmethod
    def path_is_exists(path):
        """
        Check path exists.
        :param path: A path to check
        :return: True if a path exists, False if isn't exists
        """
        return Path(path).exists()

    @staticmethod
    def get_path(old_path, *args):
        """
        Getting a new path from the old one with the addition
        of an arbitrary number of components.
        :param old_path: old_path
        :param args: an arbitrary number of components for new path
        :return: new path
        """
        return Path(old_path, *args)

    @staticmethod
    def get_date_in_correct_format(date_str):
        """
        The method gets a string containing the date and
        converts it according to known formats.
        If the conversion failed returns None.
        :param date_str: a string containing the date
        :return: a date in format %Y-%m-%d or None
        """
        for date_format in LIST_OF_DATE_FORMATS:
            try:
                date_time_obj = datetime.datetime.strptime(
                    date_str, date_format
                )
                return str(date_time_obj.date())
            except ValueError:
                # if the correct format wasn't received,
                # proceed to the next format in LIST_OF_DATE_FORMATS
                pass

        return None


class DataManagerInStorageAfterParsing(StorageManager):
    """
    A class that works with the storage when the program
    is running in the normal news parsing mode.
    Here the news is processed and added to the storage.
    """

    def __init__(self, source, *, data, verbose, colorize):
        """
        Init DataManagerInStorageAfterParsing
        :param source: news source
        :param verbose: verbose mode
        :param data: data that need to write to the storage
        """
        super().__init__(source, data=data, verbose=verbose, colorize=colorize)

    def get_file_name(self, date):
        """
        The method that gets the correct date of creation of the news
        and the source of the news and generates
        the file name by which the news will be saved.
        :param date: the correct date of creation of the news
        :return: file name
        """
        source = re.sub(r"\W", "_", self.source)
        file_name = f'{date}_{source}.json'
        return file_name

    def make_dir_by_key(self, data_dict):
        """
        Creating folders in which news will be stored.
        :param data_dict: dictionary with news
        """
        self.make_dir(self._get_abspath_to_storage())
        for key in data_dict.keys():
            path = self._get_abspath_to_storage()

            # key[:7] it is the year and the month in the format: 2021-10
            path = self.get_path(path, key[:7])
            self.make_dir(path)

            path = self.get_path(path, key)
            self.make_dir(path)

    def control_of_exist(self, data_dict, channel_data):
        """
        A method that checks if the file already exists.
        If the file exists, it runs the _write_or_update_data method
        with the `update' flag,
        if there is no file, it runs the _write_or_update_data method
        with the `write' flag.
        :param data_dict: dictionary with news
        :param channel_data: channel data
        """
        for date, list_of_news in data_dict.items():
            file_name = self.get_file_name(date)
            path = self._get_abspath_to_storage()
            path = self.get_path(path, date[:7], date, file_name)

            if self.path_is_exists(path):
                self._write_or_update_data(
                    path, channel_data, list_of_news, "update"
                )
            else:
                self._write_or_update_data(
                    path, channel_data, list_of_news, "write"
                )

    def _write_or_update_data(self, path, channel_data, list_of_news, flag):
        """
        If the flag is set to 'update', it reads data from the file
        and writes to the beginning of the file only those
        data that are not yet in the file.
        If the flag is set to 'write', it writes data to a file.
        :param path: the path to the file
        :param channel_data: channel data
        :param list_of_news: list of news
        :param flag: 'update' or 'write'
        """
        if flag == "update":
            data_from_file = self.read_from_storage(path)
            data_to_file = []

            for news in list_of_news:
                if news not in data_from_file:
                    data_to_file.append(news)

            # data_from_file[:1] - channel data
            data_to_file = (
                    data_from_file[:1] + data_to_file + data_from_file[1:]
            )
            self.write_to_storage(path, data_to_file)
        elif flag == "write":
            data_to_file = channel_data + list_of_news
            self.write_to_storage(path, data_to_file)

    def split_data_by_news(self):
        """
        A method that splits the data received from the site by dates
        and writes it to the dictionary,
        where the key is the date and the value is a list of news.
        if the site uses dates in an unsupported format,
        it throws an error and returns None
        :return: (channel_data, dict_for_data_saving) or None
        """
        dict_for_data_saving = {}
        channel_data, data = self.data[:1], self.data[1:]

        for news in data:
            date_in_correct_format = self.get_date_in_correct_format(
                news["date"]
            )

            if date_in_correct_format is None:
                error_print(
                    f"The site {self.source} uses an unsupported"
                    f" date format. Storage data has failed."
                )
                if self.verbose:
                    info_print(
                        "Supported date format:\n\t"
                        "{}".format('\n\t'.join(LIST_OF_DATE_FORMATS))
                    )
                return None

            dict_for_data_saving[date_in_correct_format] = dict_for_data_saving.get(
                date_in_correct_format, []
            ) + [news]

        return channel_data, dict_for_data_saving


class FindManagerWhenEnterDate(StorageManager):
    """
    A class that implements data search by date
    """

    def __init__(self, source, *, date, verbose, json_flag, limit, colorize):
        """
        Init FindManagerWhenEnterDate
        :param source: news source
        :param verbose: verbose mode
        :param date: the date on which you need to receive the news
        :param json_flag: --json value
        :param limit: --limit value
        """
        self.json_flag = json_flag
        self.limit = limit
        super().__init__(source, date=date, verbose=verbose, colorize=colorize)

    def get_content_by_paths(self, paths):
        """
        A method that has received a list of paths corresponding
        to the date and collects news from files in this paths.
        :param paths: a list of paths
        :return: a list of dictionary with news
        """
        list_of_content = []
        for path in paths:
            data_from_file = self.read_from_storage(path)
            list_of_content.append(data_from_file)

        return list_of_content

    def slice_content_by_limit(self, list_of_content):
        """
        The method that slice the list of news
        by the limit set by the user.
        :param list_of_content: a list of dictionary with news
        :return: a list of dictionary with news
        """
        if self.limit is None:
            if self.verbose:
                list_of_sources = self.get_sources_from_data(list_of_content)
                info_print(
                    "According to the entered date, news was "
                    "received from the following sources: {}".format(
                        ", ".join(list_of_sources)
                    )
                )
            return list_of_content
        if self.limit <= 0:
            error_print(
                "The limit is less than or equal to 0, "
                "news cannot be printed."
            )
            return False

        limit = self.limit
        new_list_of_content = []

        for data in list_of_content:
            channel_data, news = data[:1], data[1:]

            if limit < len(news):
                news = news[:limit]

                if news:
                    new_list_of_content.append(channel_data + news)
                break
            else:
                limit -= len(news)
                new_list_of_content.append(channel_data + news)

        if self.verbose:
            list_of_sources = self.get_sources_from_data(new_list_of_content)
            info_print(
                "According to the entered date, news was "
                "received from the following sources: {}".format(
                    ", ".join(list_of_sources)
                )
            )

        return new_list_of_content

    def check_news_by_date(self):
        """
        A method that verifies that there is news
        in the repository by the received date
        :return: False or list of paths
        """
        date_in_correct_format = self.get_date_in_correct_format(self.date)

        if date_in_correct_format is None:
            error_print(
                f"{self.date!r} is an incorrect date. "
                f"Please try to enter the date in a correct format"
            )
            return False

        path = self.get_path(
            self._get_abspath_to_storage(), date_in_correct_format[:7]
        )

        if self.path_is_exists(path):
            path = self.get_path(path, date_in_correct_format)

            if self.path_is_exists(path):
                paths = Path(path).glob(f"{date_in_correct_format}*")
                paths = list(map(str, paths))
                return paths
            else:
                self.news_was_not_founded(date_in_correct_format)
                return False
        else:
            self.news_was_not_founded(date_in_correct_format)
            return False

    def data_output(self, data):
        """
        Output information on the terminal in JSON or standard format
        :param data: list with news
        """
        if self.json_flag:
            if len(data) == 1 and isinstance(data[0], list):
                data = data[0]
            if self.verbose:
                info_print("News will be printed in JSON format")
            console_json_output(data)
        else:
            if self.verbose:
                info_print("News will be printed in a standard format")
            for feed in data:
                console_output_feed(feed, self.colorize)

    def news_was_not_founded(self, date=None):
        """
        Error output when news by date is not found.
        :param date: date entered by the user
        """
        if self.source is not None:
            error_print(
                f"No news was founded for this date: "
                f"{self.date}, and this source: {self.source}"
            )
        else:
            error_print(f"No news was found for this date - {date}")

    @staticmethod
    def get_sources_from_data(list_of_data):
        """
        A method that receives a list of news and
        generates a list of sources of this news
        :param list_of_data: news list
        :return: list of news sources
        """
        list_of_sources = []
        for data in list_of_data:
            list_of_sources.append(data[0]["source"])
        return list_of_sources


class FindManagerWhenEnterDateAndSource(FindManagerWhenEnterDate):
    """
    A class that implements data search by date and source
    """

    def __init__(self, source, *, date, verbose, json_flag, limit, colorize):
        """
        Init FindManagerWhenEnterDateAndSource
        :param source: news source
        :param verbose: verbose mode
        :param date: the date on which you need to receive the news.
        :param json_flag: --json value
        :param limit: --limit value
        """
        super().__init__(
            source, date=date, verbose=verbose,
            json_flag=json_flag, limit=limit, colorize=colorize
        )

    def get_file_name(self):
        """
        A method that returns the file name by date and source
        :return: file_name
        """
        source = re.sub(r"\W", "_", self.source)
        file_name = f'{self.date}_{source}.json'
        return file_name

    def data_output(self, data):
        """
        Output information on the terminal in JSON or standard format
        :param data: list with news
        """
        if self.json_flag:
            if self.verbose:
                info_print("News will be printed in JSON format")
            console_json_output(data)
        else:
            if self.verbose:
                info_print("News will be printed in a standard format")
            console_output_feed(data, self.colorize)

    def slice_content_by_limit(self, data):
        """
        The method that slice the list of news
        by the limit set by the user.
        :param data: a list of dictionary with news
        :return: a list of dictionary with news
        """
        if self.verbose:
            info_print(
                f"The news was searched in the storage by date: "
                f"{self.date}, and source: {self.source}"
            )
        if self.limit is None:
            return data
        if self.limit <= 0:
            error_print(
                "The limit is less than or equal to 0, "
                "news cannot be printed."
            )
            return False

        limit = self.limit
        channel_data, news = data[:1], data[1:]

        if limit < len(news):
            news = news[:limit]
        new_list_of_content = channel_data + news

        return new_list_of_content



import shutil
from pathlib import Path

import pytest

from cool_project.data_storage.working_with_storage import storage_control


@pytest.fixture()
def path_to_storage():
    """A fixture that return path to storage"""
    path = Path(__file__).parent.parent.parent
    path = Path(path, "cool_project", "data_storage", "storage")
    return path


def test_storage_control_after_parse(capsys, path_to_storage):
    """test for storage_control after parse"""
    data = [
        {
            "channel_title": "Люди Onlíner",
            "source": "source"
        },
        {
            "title": "title",
            "date": "Sat, 23 Oct 1000 17:51:11 +0300",
            "link": "link",
            "author": None,
            "category": ["Здоровье"],
            "description": "description",
            "more_description": None,
            "comments": None,
            "media_object": ["media_object"],
            "extra_links": "extra_links",
            "source_feed": None,
        }
    ]
    path = Path(__file__).parent
    storage_control(
        data=data, source="sour.ce", verbose=False,
        to_html=path, to_pdf=path, colorize=False
    )
    captured = capsys.readouterr()
    assert Path(Path(path, "feed.html")).exists()
    assert Path(Path(path, "feed.pdf")).exists()
    assert Path(Path(path_to_storage, "1000-10", "1000-10-23")).exists()
    assert captured.out == f"[INFO] A feed in HTML format was saved " \
                           f"on the path: {Path(path, 'feed.html')}\n\n" \
                           f"[INFO] A feed in PDF format was saved " \
                           f"on the path: {Path(path, 'feed.pdf')}\n\n"

    data = [
        {
            "channel_title": "Люди Onlíner",
            "source": "source"
        },
        {
            "date": "Sat, 23 Oct 10 17",
        }
    ]
    path = Path(__file__).parent
    storage_control(
        data=data, source="sour.ce", verbose=False,
        to_html=None, to_pdf=None, colorize=False
    )
    captured = capsys.readouterr()
    assert captured.out == f"[ERROR] The site sour.ce uses an " \
                           f"unsupported date format. " \
                           f"Storage data has failed.\n\n"

    Path.unlink(Path(path, "feed.html"))
    Path.unlink(Path(path, "feed.pdf"))
    shutil.rmtree(Path(path_to_storage, "1000-10"))


def test_storage_control_when_enter_date(capsys, path_to_storage):
    """test for storage_control when enter date"""
    if Path(Path(path_to_storage, "1000-10")).exists():
        shutil.rmtree(Path(path_to_storage, "1000-10"))

    data = [
        {
            "channel_title": "Люди Onlíner",
            "source": "source"
        },
        {
            "title": "title",
            "date": "Sat, 23 Oct 1000 17:51:11 +0300",
            "link": "link",
            "author": None,
            "category": ["Здоровье"],
            "description": "description",
            "more_description": None,
            "comments": None,
            "media_object": ["media_object"],
            "extra_links": "extra_links",
            "source_feed": None,
        }
    ]
    path = Path(__file__).parent
    storage_control(
        date="10001023", source=None, verbose=True, to_html=None,
        to_pdf=None, limit=1, json=True, colorize=False
    )
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] No news was found for this date - " \
                           "1000-10-23\n\n"

    # make file for test
    storage_control(
        data=data, source="sour.ce", verbose=False,
        to_html=None, to_pdf=None, colorize=False
    )
    capsys.readouterr()

    path = Path(__file__).parent
    storage_control(
        date="10001023", source=None, verbose=True, to_html=path,
        to_pdf=path, limit=1, json=True, colorize=False
    )
    captured = capsys.readouterr()
    assert Path(Path(path, "feed.html")).exists()
    assert Path(Path(path, "feed.pdf")).exists()
    assert Path(Path(path_to_storage, "1000-10", "1000-10-23")).exists()
    assert captured.out == "[INFO] The news has been successfully " \
                           "extracted from the storage\n\n" \
                           "[INFO] According to the entered date, news " \
                           "was received from the following " \
                           "sources: source\n\n" \
                           "[INFO] News will be printed in JSON format\n\n" \
                           "[\n" \
                           "    {\n" \
                           '        "channel_title": "Люди Onlíner",\n' \
                           '        "source": "source"\n' \
                           "    },\n" \
                           "    {\n" \
                           '        "title": "title",\n' \
                           '        "date": "Sat, 23 Oct 1000 ' \
                           '17:51:11 +0300",\n' \
                           '        "link": "link",\n' \
                           '        "author": null,\n' \
                           '        "category": [\n' \
                           '            "Здоровье"\n' \
                           '        ],\n' \
                           '        "description": "description",\n' \
                           '        "more_description": null,\n' \
                           '        "comments": null,\n' \
                           '        "media_object": [\n' \
                           '            "media_object"\n' \
                           '        ],\n' \
                           '        "extra_links": "extra_links",\n' \
                           '        "source_feed": null\n' \
                           '    }\n' \
                           ']\n' \
                           "[INFO] Conversion to HTML started\n\n" \
                           "[INFO] Conversion to HTML ended\n\n" \
                           "[INFO] A feed in HTML format was saved " \
                           f"on the path: {Path(path, 'feed.html')}\n\n" \
                           "[INFO] Fonts have been received\n\n" \
                           "[INFO] PDF generation started\n\n" \
                           "[INFO] PDF has been generated\n\n" \
                           "[INFO] A feed in PDF format was saved " \
                           f"on the path: {Path(path, 'feed.pdf')}\n\n"

    storage_control(
        date="10001023", source=None, verbose=True, to_html=None,
        to_pdf=None, limit=0, json=True, colorize=False
    )
    captured = capsys.readouterr()
    assert captured.out == "[INFO] The news has been successfully " \
                           "extracted from the storage\n\n" \
                           "[ERROR] The limit is less than or equal " \
                           "to 0, news cannot be printed.\n\n"

    Path.unlink(Path(path, "feed.html"))
    Path.unlink(Path(path, "feed.pdf"))
    shutil.rmtree(Path(path_to_storage, "1000-10"))


def test_storage_control_when_enter_date_and_source(capsys, path_to_storage):
    """test for storage_control when enter date and source"""
    if Path(Path(path_to_storage, "1000-10")).exists():
        shutil.rmtree(Path(path_to_storage, "1000-10"))

    data = [
        {
            "channel_title": "Люди Onlíner",
            "source": "source"
        },
        {
            "title": "title",
            "date": "Sat, 23 Oct 1000 17:51:11 +0300",
            "link": "link",
            "author": None,
            "category": ["Здоровье"],
            "description": "description",
            "more_description": None,
            "comments": None,
            "media_object": ["media_object"],
            "extra_links": "extra_links",
            "source_feed": None,
        }
    ]
    path = Path(__file__).parent
    storage_control(
        date="10001023", source="sour.ce", verbose=True, to_html=None,
        to_pdf=None, limit=1, json=True, colorize=False
    )
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] No news was founded for this date:" \
                           " 10001023, and this source: sour.ce\n\n"

    # make file for test
    storage_control(
        data=data, source="sour.ce", verbose=False, to_html=None,
        to_pdf=None, colorize=False
    )
    capsys.readouterr()

    path = Path(__file__).parent
    storage_control(
        date="10001023", source="sour.ce", verbose=True, to_html=path,
        to_pdf=path, limit=1, json=True, colorize=False
    )
    captured = capsys.readouterr()
    assert Path(Path(path, "feed.html")).exists()
    assert Path(Path(path, "feed.pdf")).exists()
    assert Path(Path(path_to_storage, "1000-10", "1000-10-23")).exists()
    assert captured.out == "[INFO] The news has been successfully " \
                           "extracted from the storage\n\n" \
                           "[INFO] The news was searched in the " \
                           "storage by date: 1000-10-23, " \
                           "and source: sour.ce\n\n" \
                           "[INFO] News will be printed in JSON format\n\n" \
                           "[\n" \
                           "    {\n" \
                           '        "channel_title": "Люди Onlíner",\n' \
                           '        "source": "source"\n' \
                           "    },\n" \
                           "    {\n" \
                           '        "title": "title",\n' \
                           '        "date": "Sat, 23 Oct 1000 ' \
                           '17:51:11 +0300",\n' \
                           '        "link": "link",\n' \
                           '        "author": null,\n' \
                           '        "category": [\n' \
                           '            "Здоровье"\n' \
                           '        ],\n' \
                           '        "description": "description",\n' \
                           '        "more_description": null,\n' \
                           '        "comments": null,\n' \
                           '        "media_object": [\n' \
                           '            "media_object"\n' \
                           '        ],\n' \
                           '        "extra_links": "extra_links",\n' \
                           '        "source_feed": null\n' \
                           '    }\n' \
                           ']\n' \
                           "[INFO] Conversion to HTML started\n\n" \
                           "[INFO] Conversion to HTML ended\n\n" \
                           "[INFO] A feed in HTML format was saved " \
                           f"on the path: {Path(path, 'feed.html')}\n\n" \
                           "[INFO] Fonts have been received\n\n" \
                           "[INFO] PDF generation started\n\n" \
                           "[INFO] PDF has been generated\n\n" \
                           "[INFO] A feed in PDF format was saved " \
                           f"on the path: {Path(path, 'feed.pdf')}\n\n"

    storage_control(
        date="10001023", source="sour.ce", verbose=True, to_html=None,
        to_pdf=None, limit=0, json=True, colorize=False
    )
    captured = capsys.readouterr()
    assert captured.out == "[INFO] The news has been successfully " \
                           "extracted from the storage\n\n" \
                           "[INFO] The news was searched in the " \
                           "storage by date: 1000-10-23, " \
                           "and source: sour.ce\n\n" \
                           "[ERROR] The limit is less than or equal " \
                           "to 0, news cannot be printed.\n\n"

    p = Path(
        path_to_storage, "1000-10", "1000-10-23", "1000-10-23_sour_ce.json"
    )
    target = Path(
        path_to_storage, "1000-10", "1000-10-23", "1000-10-23_sou3_ce.json"
    )
    p.rename(target)
    storage_control(
        date="10001023", source="sour.ce", verbose=True, to_html=None,
        to_pdf=None, limit=1, json=True, colorize=False
    )
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] No news was founded for this date" \
                           ": 1000-10-23, and this " \
                           "source: sour.ce\n\n"

    Path.unlink(Path(path, "feed.html"))
    Path.unlink(Path(path, "feed.pdf"))
    shutil.rmtree(Path(path_to_storage, "1000-10"))



import pytest

from rss_reader import RSSParser, start_parsing
from project_settings import PROGRAM_VERSION


@pytest.mark.parametrize("args, expected",
                         [("source --json --verbose --limit=2 "
                           "--date=2020 --to-html=path --to-pdf=path",
                           ("source", True, True, 2, "2020", "path", "path")),
                          (" ",
                           (None, False, False, None, None, None, None)),
                          ("source --verbose",
                           ("source", False, True, None, None, None, None)),
                          ("--json --verbose --limit=2 --date=2020",
                           (None, True, True, 2, "2020", None, None)),
                          ("source --json --verbose --limit=2 --date=2020",
                           ("source", True, True, 2, "2020", None, None)),
                          ("source --verbose --limit=2 "
                           "--date=2020 --to-html=path",
                           ("source", False, True, 2, "2020", "path", None)),
                          ("source --json --verbose --limit=2 "
                           "--date=2020 --to-pdf=path",
                           ("source", True, True, 2, "2020", None, "path")),
                          ("--json --verbose --limit=2 --date=2020 "
                           "--to-html=path --to-pdf=path",
                           (None, True, True, 2, "2020", "path", "path")),
                          ]
                         )
def test_RSSParser_method_init_argparse(args, expected):
    """test for method init_argparse"""
    parser = RSSParser(args.split())
    actual = (
        parser.source, parser.json, parser.verbose,
        parser.limit, parser.date, parser.to_html, parser.to_pdf
    )
    assert actual == expected


@pytest.mark.parametrize("args, expected",
                         [("--version",
                           f"Version {PROGRAM_VERSION}\n"),
                          ("source --json --verbose --limit=2 --date=2020 "
                           "--to-html=path --to-pdf=path --version",
                           f"Version {PROGRAM_VERSION}\n"),
                          ]
                         )
def test_RSSParser_method_init_argparse(args, expected, capsys):
    """test for method init_argparse"""
    try:
        parser = RSSParser(["--version"])
        captured = capsys.readouterr()
    except SystemExit:
        captured = capsys.readouterr()
    finally:
        assert captured.out == expected


def test_RSSParser_method_parsing(capsys):
    """test for method parsing"""
    parser = RSSParser("https://people.onliner.by/feed --limit=1".split())
    parser.parsing()
    captured = capsys.readouterr()
    assert captured.out == "[INFO] Receiving the news was successful\n\n"

    parser = RSSParser("https://people.onliner.by --limit=1".split())
    parser.parsing()
    captured = capsys.readouterr()
    assert captured.out == "[WARNING] 'https://people.onliner.by' " \
                           "isn't a RSS. Please try to " \
                           "enter a correct URL\n\n" \
                           "[ERROR] Data wasn't received\n\n"


class Response:
    """The class which simulates the response.status_code"""
    def __init__(self, value):
        self.status_code = value


def test_RSSParser_method_isvalid(capsys):
    """test for method isvalid"""
    parser = RSSParser("https://people.onliner.by/feed --limit=1".split())
    actual = parser._isvalid(Response(200))
    assert actual

    parser = RSSParser(
        "https://people.onliner.by/feed --limit=1 --verbose".split()
    )
    parser._isvalid(None)
    captured = capsys.readouterr()
    assert captured.out == "[INFO] The program stop running with error," \
                           " when it try to get information " \
                           "from 'https://people.onliner.by/feed'\n\n"

    parser = RSSParser("https://people.onliner.by/feed --limit=1".split())
    parser._isvalid(Response(404))
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] 'https://people.onliner.by/feed': " \
                           "404 Page Not Found\n\n"


def test_RSSParser_method_get_html():
    """test for method get_html"""
    parser = RSSParser("https://people.onliner.by/feed --limit=1".split())
    response = parser._get_html()
    assert response.status_code == 200


def test_RSSParser_method_check_error_status_code(capsys):
    """test for method check_error_status_code"""
    parser = RSSParser("https://people.onliner.by/feed --limit=1".split())
    parser._check_error_status_code(405)
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] Error seems to have been caused by" \
                           " the client. Check url which you give.\n\n"

    parser._check_error_status_code(500)
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] The server failed to " \
                           "execute a request\n\n"

    parser._check_error_status_code(700)
    captured = capsys.readouterr()
    assert captured.out == "[ERROR] Error which can't be processed" \
                           " because status code don't defined\n\n"


def test_RSSParser_method_check_date_and_source(capsys):
    """test for method check_date_and_source"""
    parser = RSSParser("https://people.onliner.by/feed --limit=1".split())
    actual = parser.check_date_and_source()
    assert actual

    parser = RSSParser("--limit=1 --verbose".split())
    actual = parser.check_date_and_source()
    captured = capsys.readouterr()
    assert captured.out == "[INFO] Source is None\n\n" \
                           "[ERROR] A source wasn't enter\n\n"


def test_RSSParser_method_print_data_in_console(capsys):
    """test for method print_data_in_console"""
    parser = RSSParser(
        "https://people.onliner.by/feed --limit=1 --json --verbose".split()
    )
    parser.serializable_data = None
    actual = parser.print_data_in_console()
    assert not actual

    parser = RSSParser(
        "https://people.onliner.by/feed --limit=1 --json --verbose".split()
    )
    parser.serializable_data = [{"data": "data"}]
    parser.print_data_in_console()
    captured = capsys.readouterr()
    assert captured.out == "[INFO] Output news in JSON format\n\n" \
                           '[\n' \
                           '    {\n' \
                           '        "data": "data"\n' \
                           '    }\n' \
                           ']\n'
